/*
 *  Copyright (C) 2003  Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gul-toggle-toolbutton.h"
#include "gul-gobject-misc.h"
#include "galeon-marshal.h"

#include <gtk/gtktogglebutton.h>
#include <libgnome/gnome-i18n.h>

#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
//#define DEBUG_MSG(x) g_print x
#define DEBUG_MSG(x)

/**
 * Private data
 */
struct _GulToolbuttonPrivate
{
};

static void		gul_toggle_toolbutton_init			(GulToggleToolbutton *button);
static void		gul_toggle_toolbutton_class_init		(GulToggleToolbuttonClass *klass);
static void		gul_toggle_toolbutton_finalize_impl		(GObject *o);

enum GulToggleToolbuttonSignalsEnum {
	GUL_TOGGLE_TOOLBUTTON_TOGGLED,
	GUL_TOGGLE_TOOLBUTTON_LAST_SIGNAL
};
static gint GulToggleToolbuttonSignals[GUL_TOGGLE_TOOLBUTTON_LAST_SIGNAL];

static GObjectClass *parent_class = NULL;


MAKE_GET_TYPE (gul_toggle_toolbutton, "GulToggleToolbutton", GulToggleToolbutton, gul_toggle_toolbutton_class_init, 
	       gul_toggle_toolbutton_init, GUL_TYPE_TOOLBUTTON);



static void
gul_toggle_toolbutton_class_init (GulToggleToolbuttonClass *klass)
{
	GObjectClass *object_class;
	EggToolButtonClass *toolbutton_class;

	toolbutton_class = (EggToolButtonClass *)klass;

	parent_class = g_type_class_peek_parent (klass);
	
	object_class = (GObjectClass *) klass;
	
	object_class->finalize = gul_toggle_toolbutton_finalize_impl;

	toolbutton_class->button_type = GTK_TYPE_TOGGLE_BUTTON;
	
	GulToggleToolbuttonSignals[GUL_TOGGLE_TOOLBUTTON_TOGGLED] = g_signal_new (
		"toggled", G_OBJECT_CLASS_TYPE (klass),  
		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
		G_STRUCT_OFFSET (GulToggleToolbuttonClass, toggled), 
		NULL, NULL, 
		galeon_marshal_VOID__VOID,
		G_TYPE_NONE, 0);
}

static void
gul_toggle_toolbutton_button_toggled (GtkWidget *widget,
				      GulToggleToolbutton *toggle_tool_button)
{
	g_signal_emit (G_OBJECT (toggle_tool_button), GulToggleToolbuttonSignals[GUL_TOGGLE_TOOLBUTTON_TOGGLED], 0);
}

static void
gul_toggle_toolbutton_init (GulToggleToolbutton *button)
{
	g_signal_connect_object (EGG_TOOL_BUTTON (button)->button, "toggled",
				 G_CALLBACK (gul_toggle_toolbutton_button_toggled), button, 0);
}

static void
gul_toggle_toolbutton_finalize_impl (GObject *o)
{
	G_OBJECT_CLASS (parent_class)->finalize (o);
}

GulToggleToolbutton *
gul_toggle_toolbutton_new (void)
{
	GulToggleToolbutton *ret = g_object_new (GUL_TYPE_TOGGLE_TOOLBUTTON, NULL);
	
	return ret;
}

GulToggleToolbutton *
gul_toggle_toolbutton_new_from_stock (const gchar *stock_id)
{
	GulToggleToolbutton *ret;
	
	g_return_val_if_fail (stock_id != NULL, NULL);
	
	ret = g_object_new (GUL_TYPE_TOGGLE_TOOLBUTTON,
			    "stock_id", stock_id,
			    NULL);
  	return ret;
}

gboolean
gul_toggle_toolbutton_get_toggled (GulToggleToolbutton *b)
{
	GtkToggleButton *tb = GTK_TOGGLE_BUTTON (gul_toolbutton_get_button (GUL_TOOLBUTTON (b)));
	return gtk_toggle_button_get_active (tb);
}

void
gul_toggle_toolbutton_set_toggled (GulToggleToolbutton *b, gboolean value)
{
	GtkToggleButton *tb = GTK_TOGGLE_BUTTON (gul_toolbutton_get_button (GUL_TOOLBUTTON (b)));
	gtk_toggle_button_set_active (tb, value);
}

